package ver4;

import java.time.LocalDate;
import java.time.LocalTime;
import java.util.ArrayList;
import java.util.List;
import java.util.TreeMap;

public class Course {
	private String department;
	private String number;
	protected String name;
	private TreeMap<String,Registration> registrations;
	private int numRegistrations;
	private int numSeats;
	private int numSeatsAvailable;
	private boolean isFull;
	
	protected Course() {}
	
	public Course(String department, String number, int numSeats) {
		this.department = department;
		this.number = number;
		this.name = department + " " + number;
		this.numSeats = numSeats;
		this.numSeatsAvailable = numSeats;
		this.isFull = false;
		this.registrations = new TreeMap<>();
	}

	Course(String name) {
		this.name = name;
	}

	public String getDepartment() {
		return department;
	}

	public String getNumber() {
		return number;
	}

	public String getName() {
		return name;
	}

	public int getNumSeats() {
		return numSeats;
	}

	public int getNumSeatsAvailable() {
		return numSeatsAvailable;
	}

	public boolean isFull() {
		return isFull;
	}

	public int getNumRegistrations() {
		return registrations.size();
	}
	
	public boolean addRegistration(Registration r) {
		if(registrations.containsKey(r.getStudent().getSsn())) {
			return false;
		}
		if(isFull()) {
			return false;
		}
		registrations.put(r.getStudent().getSsn(),r);
		numSeatsAvailable--;
		if(numSeatsAvailable==0) {
			isFull = true;
		}
		return true;
	}
	
	public boolean equals(Object o) {
		if(o instanceof Course c) {
			return this.name.equals(c.name);
		}
		return false;
	}
	
	public boolean setGrades(Student s, double hwAvg, double testAvg, double finalExam) {
		if(!registrations.containsKey(s.getSsn())) {
			return false;
		}
		Registration r = registrations.get(s.getSsn());
		r.setGrades(hwAvg, testAvg, finalExam);
		return true;
	}
	
	public boolean isRegistered(Student s) {
		return registrations.containsKey(s.getSsn());
	}
	
	public Student getStudent(String ssn) {
		if(registrations.containsKey(ssn)) {
			Registration r = registrations.get(ssn);
			return r.getStudent();
		}
		return null;
	}
	
	@Override
	public String toString() {
		return "Course [department=" + department + 
				", number=" + number + 
				", name=" + name + 
				", numRegistrations=" + numRegistrations + 
				", numSeats=" + numSeats + 
				", numSeatsAvailable=" + numSeatsAvailable + 
				", isFull=" + isFull + "]";
	}
	
	public static void main(String[] args) {
		LocalDate bDate = LocalDate.parse("1999-04-26");
		Student s1 = new Student("Dave", "Gibson", bDate, "222348834");
		Course c1 = new Course("CS", "3326", 2);
		Registration r = new Registration(s1,c1);
		c1.addRegistration(r);
		Student s2 = new Student("Pina", "Cooley", bDate, "435758932");
		Registration r2 = new Registration(s2,c1);
		c1.addRegistration(r2);
		Student sActual = c1.getStudent("3");
	}
	
}
