package ver4;

import java.time.LocalDate;
import java.util.List;
import java.util.TreeMap;

public class Registrar {
	
	private TreeMap<String,Student> students;
	private TreeMap<String,Course> courses;

	public Registrar() {
		this.students = new TreeMap<>();
		this.courses = new TreeMap<>();
	}

	public int getNumStudents() {
		return students.size();
	}
	
	public int getNumCourses() {
		return courses.size();
	}
	
	public Student addStudent(String firstName, String lastName, LocalDate birthDate, String ssn) {
		Student s = new Student(firstName, lastName, birthDate, ssn);
		if(!students.containsKey(ssn)) {
			students.put(ssn, s);
			return s;
		}
		return null;
	}

	public Course addCourse(String department, String number, int numSeats) {
		Course c = new Course(department, number, numSeats);
		if(!courses.containsKey(c.getName())) {
			courses.put(c.getName(), c);
			return c;
		}
		return null;
	}
	
	public boolean register(Student s, Course c) {
		// Registrar must contain reference to student and course before registering,
		// and student must not already be registered for course.
		if(!students.containsKey(s.getSsn()) ||
		   !courses.containsKey(c.getName()) ||
		   s.isRegisteredFor(c) ) {
			return false;
		}
		Registration r = new Registration(s,c);
		s.addRegistration(r);
		c.addRegistration(r);
		return true;
	}
	
	public Student getStudent(String ssn) {
		return students.get(ssn);
	}
	
	public Course getCourse(String name) {
		return courses.get(name);
	}
}
