package ver4;

import java.time.LocalDate;
import java.util.ArrayList;
import java.util.Collections;
import java.util.List;
import java.util.TreeMap;

public class Student {
	private String firstName;
	private String lastName;
	private LocalDate birthDate;
	private String ssn;
	private TreeMap<String,Registration> registrations;
	
	protected Student() {}
	
	public Student(String firstName, String lastName, LocalDate birthDate, String ssn) {
		this.firstName = firstName;
		this.lastName = lastName;
		this.birthDate = birthDate;
		this.ssn = ssn;
		this.registrations = new TreeMap<>();
	}
	
	Student(String ssn) {
		this.ssn = ssn;
	}

	public String getFirstName() {
		return firstName;
	}

	public String getLastName() {
		return lastName;
	}

	public LocalDate getBirthDate() {
		return birthDate;
	}

	public String getSsn() {
		return ssn;
	}

	public int getNumRegistrations() {
		return registrations.size();
	}
	
	public boolean addRegistration(Registration r) {
		if(registrations.containsKey(r.getCourseName())) {
			return false;
		}
		registrations.put(r.getCourseName(),r);
		return true;
	}
	
	public boolean equals(Object o) {
		if(o instanceof Student s) {
			return this.ssn.equals(s.ssn);
		}
		return false;
	}
	
	// Added for use by StudentReportGenerator.
	// In practice, may or may not need to be unmodifiable
	List<Registration> getRegistrations() {
		List<Registration> list = new ArrayList<>(registrations.values());
		return Collections.unmodifiableList(list);
	}
	
	public boolean isRegisteredFor(Course c) {
		return registrations.containsKey(c.getName());
	}
	
	public Course getCourse(String name) {
		if(registrations.containsKey(name)) {
			return registrations.get(name).getCourse();
		}
		return null;
	}
	

	@Override
	public String toString() {
		return "Student [firstName=" + firstName + ", lastName=" + lastName + ", birthDate=" + birthDate + ", ssn="
				+ ssn + "]";
	}
	
	public static void main(String[] args) {
		LocalDate bDate = LocalDate.parse("1999-04-26");
		Student s1 = new Student("Dave", "Gibson", bDate, "222348834");
		Course c1 = new Course("CS", "3326", 3);
		Registration r = new Registration(s1,c1);
		s1.addRegistration(r);
		Course c2 = new Course("CS", "4432", 5);
		r = new Registration(s1,c2);
		List<Registration> registrations = s1.getRegistrations();
		registrations.remove(0);
	}
	
}
