package ver4_test;

import static org.junit.jupiter.api.Assertions.*;
import java.time.LocalDate;
import org.junit.jupiter.api.Test;

import ver4.Course;
import ver4.Registration;
import ver4.Student;

class CourseTest {

	@Test
	void testEqualsSucceed() {
		Course c1 = new Course("CS", "3326", 3);
		Course c2 = new Course("CS", "3326", 53);
		assertTrue(c1.equals(c2));
	}
	
	@Test
	void testEqualsFailWithCourseObject() {
		Course c1 = new Course("CS", "3326", 3);
		Course c2 = new Course("CS", "1111", 53);
		assertFalse(c1.equals(c2));
	}

	@Test
	void testEqualsFailWithOtherObject() {
		Course c1 = new Course("CS", "3326", 3);
		assertFalse(c1.equals(new String("Not a Course")));
	}

	@Test
	void testAddRegistration() {
		String date = "1999-04-26";
		LocalDate bDate = LocalDate.parse(date);
		Student s1 = new Student("Dave", "Gibson", bDate, "222348834");
		Course c1 = new Course("CS", "3326", 3);
		Registration r = new Registration(s1, c1);
		boolean wasAdded = c1.addRegistration(r);
		assertAll("Should add Registration",
			    () -> assertFalse(c1.isFull()),
			    () -> assertEquals(2, c1.getNumSeatsAvailable()),
			    () -> assertTrue(wasAdded)
			    );
		
	}
	
	@Test
	void testAddRegistrationMultiple() {
		String date = "1999-04-26";
		LocalDate bDate = LocalDate.parse(date);
		Student s1 = new Student("Dave", "Gibson", bDate, "222348834");
		Course c1 = new Course("CS", "3326", 3);
		Registration r1 = new Registration(s1, c1);
		boolean wasAdded1 = c1.addRegistration(r1);
		Student s2 = new Student("Paul", "Gibson", bDate, "444348834");
		Registration r2 = new Registration(s2, c1);
		boolean wasAdded2 = c1.addRegistration(r2);
		
		assertAll("Should add Registration",
			    () -> assertFalse(c1.isFull()),
			    () -> assertEquals(1, c1.getNumSeatsAvailable()),
			    () -> assertTrue(wasAdded1),
			    () -> assertTrue(wasAdded2)
				);
		
	}
	
	@Test
	void testAddRegistrationFull() {
		String date = "1999-04-26";
		LocalDate bDate = LocalDate.parse(date);
		Student s1 = new Student("Dave", "Gibson", bDate, "222348834");
		Course c1 = new Course("CS", "3326", 3);
		Registration r1 = new Registration(s1, c1);
		c1.addRegistration(r1);

		Student s2 = new Student("Paul", "Gibson", bDate, "444348834");
		Registration r2 = new Registration(s2, c1);
		c1.addRegistration(r2);

		Student s3 = new Student("Steve", "Dobbs", bDate, "666348834");
		Registration r3 = new Registration(s3, c1);
		c1.addRegistration(r3);
		
		Student s4 = new Student("Alice", "Glass", bDate, "777348834");
		Registration r4 = new Registration(s4, c1);
		boolean wasAdded = c1.addRegistration(r4);
		
		assertAll("Should not add Registration",
			    () -> assertTrue(c1.isFull()),
			    () -> assertEquals(0, c1.getNumSeatsAvailable()),
			    () -> assertFalse(wasAdded)
				);
		
	}

	@Test
	void testGetStudent() {
		String date = "1999-04-26";
		LocalDate bDate = LocalDate.parse(date);
		Student sExpected = new Student("Dave", "Gibson", bDate, "222348834");
		Course c1 = new Course("CS", "3326", 3);
		Registration r1 = new Registration(sExpected, c1);
		c1.addRegistration(r1);
		Student sActual = c1.getStudent("222348834");
		assertEquals(sExpected, sActual);
	}

	@Test
	void testGetStudentFail() {
		String date = "1999-04-26";
		LocalDate bDate = LocalDate.parse(date);
		Student sExpected = new Student("Dave", "Gibson", bDate, "222348834");
		Course c1 = new Course("CS", "3326", 3);
		Registration r1 = new Registration(sExpected, c1);
		c1.addRegistration(r1);
		Student sActual = c1.getStudent("111111111");
		assertNull(sActual);
	}

}
