package ver4_test;

import static org.junit.jupiter.api.Assertions.*;

import java.time.LocalDate;

import org.junit.jupiter.api.Test;

import ver4.Registrar;
import ver4.Student;
import ver4.Course;

class RegistrarTest {

	@Test
	void testAddStudentSucceed() {
		Registrar registrar = new Registrar();
		LocalDate bDate = LocalDate.parse("1999-04-26");
		Student sExpected = new Student("Dave", "Gibson", bDate, "222348834");
		Student sActual = registrar.addStudent("Dave", "Gibson", bDate, "222348834");
	
		assertAll("Should add Student",
			    () -> assertEquals(sExpected, sActual),
			    () -> assertEquals(1, registrar.getNumStudents())
				);
	}

	@Test
	void testAddStudentFail() {
		Registrar registrar = new Registrar();
		Student sExpected = null;
		LocalDate bDate = LocalDate.parse("1999-04-26");
		Student sFirst = registrar.addStudent("Dave", "Gibson", bDate, "222348834");
		Student sActual = registrar.addStudent("Dave", "Gibson", bDate, "222348834");
	
		assertAll("Should not add Student",
			    () -> assertEquals(sExpected, sActual),
			    () -> assertEquals(1, registrar.getNumStudents())
				);
	}

	@Test
	void testAddCourseSucceed() {
		Course cExpected = new Course("CS", "3326", 3);
		Registrar registrar = new Registrar();
		Course cActual = registrar.addCourse("CS", "3326", 3);
		
		assertAll("Should add Course",
			    () -> assertEquals(cExpected, cActual),
			    () -> assertEquals(1, registrar.getNumCourses())
				);
	}

	@Test
	void testAddCourseFail() {
		Course cExpected = null;
		Registrar registrar = new Registrar();
		Course cFirst = registrar.addCourse("CS", "3326", 3);
		Course cActual = registrar.addCourse("CS", "3326", 3);
		
		assertAll("Should not add Course",
			    () -> assertEquals(cExpected, cActual),
			    () -> assertEquals(1, registrar.getNumCourses())
				);
	}

	@Test
	void testRegisterSucceed() {
		Registrar registrar = new Registrar();
		LocalDate bDate = LocalDate.parse("1999-04-26");
		Student student = registrar.addStudent("Dave", "Gibson", bDate, "222348834");
		Course course = registrar.addCourse("CS", "3326", 3);
		boolean isRegistered = registrar.register(student, course);
		assertTrue(isRegistered);
	}

	@Test
	void testRegisterFail() {
		Registrar registrar = new Registrar();
		LocalDate bDate = LocalDate.parse("1999-04-26");
		Student student = registrar.addStudent("Dave", "Gibson", bDate, "222348834");
		Course course = registrar.addCourse("CS", "3326", 3);
		boolean isRegistered = registrar.register(student, course);
		isRegistered = registrar.register(student, course);
		assertFalse(isRegistered);
	}

	@Test
	void testGetStudentSucceed() {
		Registrar registrar = new Registrar();
		LocalDate bDate = LocalDate.parse("1999-04-26");
		Student expectedStudent = registrar.addStudent("Dave", "Gibson", bDate, "222348834");
		Student actualStudent = registrar.getStudent("222348834");
		assertEquals(expectedStudent, actualStudent);
	}

	@Test
	void testGetStudentFail() {
		Registrar registrar = new Registrar();
		LocalDate bDate = LocalDate.parse("1999-04-26");
		Student expectedStudent = registrar.addStudent("Dave", "Gibson", bDate, "222348834");
		Student actualStudent = registrar.getStudent("123456789");
		assertNotEquals(expectedStudent, actualStudent);
	}

	@Test
	void testGetCourseSucceed() {
		Registrar registrar = new Registrar();
		Course expectedCourse = registrar.addCourse("CS", "3326", 3);
		Course actualCourse = registrar.getCourse("CS 3326");
		assertEquals(expectedCourse, actualCourse);
	}

	@Test
	void testGetCourseFail() {
		Registrar registrar = new Registrar();
		Course expectedCourse = registrar.addCourse("CS", "3326", 3);
		Course actualCourse = registrar.getCourse("CS 1111");
		assertNotEquals(expectedCourse, actualCourse);
	}

}
