package ver4_test;

import static org.junit.jupiter.api.Assertions.*;

import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;
import java.time.LocalDate;
import org.junit.jupiter.api.Test;

import ver4.Course;
import ver4.Registration;
import ver4.Student;

class RegistrationTest {

	@Test
	void testEqualsSucceed() {
		String date = "1999-04-26";
		LocalDate bDate = LocalDate.parse(date);
		Student s1 = new Student("Dave", "Gibson", bDate, "222348834");
		Course c1 = new Course("CS", "3326", 3);
		Registration rExpected = new Registration(s1, c1);

		Student s2 = new Student("Walter", "Jensen", bDate, "222348834");
		Course c2 = new Course("CS", "3326", 99);
		Registration rActual = new Registration(s2, c2);

		assertEquals(rExpected, rActual);
	}

	@Test
	void testEqualsFailByCourse() {
		String date = "1999-04-26";
		LocalDate bDate = LocalDate.parse(date);
		Student s1 = new Student("Dave", "Gibson", bDate, "222348834");
		Course c1 = new Course("CS", "3326", 3);
		Registration rExpected = new Registration(s1, c1);

		Student s2 = new Student("Walter", "Jensen", bDate, "222348834");
		Course c2 = new Course("CS", "1111", 99);
		Registration rActual = new Registration(s2, c2);

		assertNotEquals(rExpected, rActual);
	}

	@Test
	void testEqualsFailByStudent() {
		String date = "1999-04-26";
		LocalDate bDate = LocalDate.parse(date);
		Student s1 = new Student("Dave", "Gibson", bDate, "222348834");
		Course c1 = new Course("CS", "3326", 3);
		Registration rExpected = new Registration(s1, c1);

		Student s2 = new Student("Walter", "Jensen", bDate, "111111111");
		Course c2 = new Course("CS", "3326", 99);
		Registration rActual = new Registration(s2, c2);

		assertNotEquals(rExpected, rActual);
	}

	// Uses reflection to access package level method, setGrades
	@Test
	void testSetGrades() throws NoSuchMethodException, IllegalAccessException, InvocationTargetException {
		String date = "1999-04-26";
		LocalDate bDate = LocalDate.parse(date);
		Student s1 = new Student("Dave", "Gibson", bDate, "222348834");
		Course c1 = new Course("CS", "3326", 3);
		Registration r = new Registration(s1, c1);
		
		Method m = getSetGradesMethod();
		m.invoke(r, 90,  80, 70);
		double actualFinalAverage = r.getFinalAverage();
		double expectedFinalAverge = 0.2* 90 + 0.6 * 80 + 0.2*70;
		assertEquals(expectedFinalAverge, actualFinalAverage);
	}
	
	private Method getSetGradesMethod() throws NoSuchMethodException {
	    Method method = Registration.class.getDeclaredMethod("setGrades", double.class, double.class, double.class);
	    method.setAccessible(true);
	    return method;
	}

}
