package ver4_test;

import static org.junit.jupiter.api.Assertions.*;
import java.time.LocalDate;
import org.junit.jupiter.api.Test;

import ver4.Course;
import ver4.Registration;
import ver4.Student;

class StudentTest {

	@Test
	void testEqualsSucceed() {
		LocalDate bDate = LocalDate.parse("1999-04-26");
		Student student1 = new Student("Dave", "Gibson", bDate, "222348834");
		bDate = LocalDate.parse("2003-11-30");
		Student student2 = new Student("Art", "Taylor", bDate, "222348834");
		assertTrue(student1.equals(student2));
	}
	
	@Test
	void testEqualsFailWithStudentObject() {
		LocalDate bDate = LocalDate.parse("1999-04-26");
		Student student1 = new Student("Dave", "Gibson", bDate, "222348834");
		bDate = LocalDate.parse("2003-11-30");
		Student student2 = new Student("Art", "Taylor", bDate, "922348834");
		assertFalse(student1.equals(student2));
	}

	@Test
	void testEqualsFailWithOtherObject() {
		LocalDate bDate = LocalDate.parse("1999-04-26");
		Student student1 = new Student("Dave", "Gibson", bDate, "222348834");
		assertFalse(student1.equals(new String("Not a student")));
	}

	@Test
	void testAddRegistration() {
		String date = "1999-04-26";
		LocalDate bDate = LocalDate.parse(date);
		Student s1 = new Student("Dave", "Gibson", bDate, "222348834");
		Course c1 = new Course("CS", "3326", 3);
		Registration r = new Registration(s1, c1);
		boolean wasAdded = s1.addRegistration(r);
		assertAll("Should add Registration",
			    () -> assertEquals(1, s1.getNumRegistrations()),
			    () -> assertTrue(wasAdded)
			    );
	}
	
	@Test
	void testAddRegistrationMultiple() {
		String date = "1999-04-26";
		LocalDate bDate = LocalDate.parse(date);
		Student s1 = new Student("Dave", "Gibson", bDate, "222348834");
		Course c1 = new Course("CS", "3326", 3);
		Registration r1 = new Registration(s1, c1);
		boolean wasAdded1 = s1.addRegistration(r1);

		Course c2 = new Course("CS", "4400", 4);
		Registration r2 = new Registration(s1, c2);
		boolean wasAdded2 = s1.addRegistration(r2);
		
		assertAll("Should add Registrations",
			    () -> assertEquals(2, s1.getNumRegistrations()),
			    () -> assertTrue(wasAdded1),
			    () -> assertTrue(wasAdded2)
				);
	}

	@Test
	void testAddRegistrationFail() {
		String date = "1999-04-26";
		LocalDate bDate = LocalDate.parse(date);
		Student s1 = new Student("Dave", "Gibson", bDate, "222348834");
		Course c1 = new Course("CS", "3326", 3);
		Registration r1 = new Registration(s1, c1);
		boolean wasAdded1 = s1.addRegistration(r1);

		Course c2 = new Course("CS", "3326", 4);
		Registration r2 = new Registration(s1, c2);
		boolean wasAdded2 = s1.addRegistration(r2);
		
		assertAll("Should add Registrations",
			    () -> assertEquals(1, s1.getNumRegistrations()),
			    () -> assertFalse(wasAdded2)
				);
	}

	@Test
	void testGetCourse() {
		String date = "1999-04-26";
		LocalDate bDate = LocalDate.parse(date);
		Student s1 = new Student("Dave", "Gibson", bDate, "222348834");

		Course cExpected = new Course("CS", "3326", 3);
		Registration r1 = new Registration(s1, cExpected);
		boolean wasAdded1 = s1.addRegistration(r1);

		Course c2 = new Course("CS", "4480", 4);
		Registration r2 = new Registration(s1, c2);
		boolean wasAdded2 = s1.addRegistration(r2);

		Course cActual = s1.getCourse("CS 3326");
		
		assertEquals(cExpected, cActual);
	}

	@Test
	void testGetCourseFail() {
		String date = "1999-04-26";
		LocalDate bDate = LocalDate.parse(date);
		Student s1 = new Student("Dave", "Gibson", bDate, "222348834");

		Course cExpected = new Course("CS", "3326", 3);
		Registration r1 = new Registration(s1, cExpected);
		boolean wasAdded1 = s1.addRegistration(r1);

		Course c2 = new Course("CS", "4480", 4);
		Registration r2 = new Registration(s1, c2);
		boolean wasAdded2 = s1.addRegistration(r2);

		Course cActual = s1.getCourse("CS 1234");
		
		assertNull(cActual);
	}

}
